use std::sync::Arc;

use axum::{Router, routing::{put, get}, extract::{Path, Json}, Extension, response::{IntoResponse, Response}, http::StatusCode};
use tracing::{warn, debug};

use crate::state::{types::{ElementId, ElementContent}, ApiState};




pub fn get_router(state: ApiState) -> Router {
    Router::new()
        .route("/element", put(create_element))
        .route("/element/:id", get(get_element).post(set_element).delete(remove_element))
        .layer(Extension(Arc::new(state)))
}

async fn get_element(Path(id): Path<ElementId>, s: Extension<Arc<ApiState>>) -> Response {
    let element = s.get_element(&id);
    match element {
        Ok(el) => (StatusCode::OK, Json{0: el}).into_response(),
        Err(e) => {
            warn!("Element not found:\n{:?}", e);
            StatusCode::NOT_FOUND.into_response()
        }
    }
}

async fn create_element(s: Extension<Arc<ApiState>>, Json(content): Json<ElementContent>) -> Response {
    let element_id = s.create_element(&content);
    debug!("{:?}", element_id);
    match element_id {
        Ok(id) => (StatusCode::OK, Json{0: &Into::<String>::into(&id)}).into_response(),
        Err(_) => StatusCode::INTERNAL_SERVER_ERROR.into_response(),
    }
}

async fn set_element(Path(id): Path<ElementId>, s: Extension<Arc<ApiState>>, Json(content): Json<ElementContent>) -> Response {
    let res = s.write_element_content(&id, &content);
    match res {
        Ok(_) => StatusCode::OK.into_response(),
        Err(_) => StatusCode::INTERNAL_SERVER_ERROR.into_response(),
    }
}

async fn remove_element(Path(id): Path<ElementId>, s: Extension<Arc<ApiState>>) -> Response {
    let res = s.remove_element(&id);
    match res {
        Ok(_) => StatusCode::OK.into_response(),
        Err(_) => StatusCode::INTERNAL_SERVER_ERROR.into_response(),
    }
}